﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Block
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        const int START = 0;
        const int MOVING = 1;

        Texture2D ballTexture;
        Texture2D blockTexture;
        Texture2D paddleTexture;
        Texture2D backTexture;
        SpriteFont font;

        Vector2 ballTexturePosition = new Vector2(625, 620);
        Vector2 paddleTexturePosition = new Vector2(590, 650);
        List<Vector2> blocks = new List<Vector2>();
        int gameState = START;
        Vector2 speed;
        int score = 0;

        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;

        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            audioEngine = new AudioEngine("Content/effectsound.xgs");
            waveBank = new WaveBank(audioEngine, "Content/Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, "Content/Sound Bank.xsb");

            for (int i = 0; i < 5; i++)
                for (int j = 0; j < 10; j++)
                    blocks.Add(new Vector2((60 * j) + 340, (i * 30) + 100)); 

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here

            ballTexture = Content.Load<Texture2D>("Graphics/ball");
            blockTexture = Content.Load<Texture2D>("Graphics/block");
            paddleTexture = Content.Load<Texture2D>("Graphics/paddle");
            backTexture = Content.Load<Texture2D>("Graphics/space");
            font = Content.Load<SpriteFont>("GameFont"); 

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            switch (gameState)
            {
                case START:
                    if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed)
                    {
                        score = 0;
                        ballTexturePosition.X = paddleTexturePosition.X + 35;
                        ballTexturePosition.Y = paddleTexturePosition.Y - 30;
                        blocks.Clear();
                        for (int i = 0; i < 5; i++)
                            for (int j = 0; j < 10; j++)
                                blocks.Add(new Vector2((60 * j) + 340, (i * 30) + 100));
                        speed.X = new Random().Next(4) + 2;
                        speed.Y = -(new Random().Next(3) + 2);
                        soundBank.GetCue("pon").Play();
                        gameState = MOVING;
                    }
                    break;
                case MOVING:
                    if (GamePad.GetState(PlayerIndex.One).DPad.Right == ButtonState.Pressed)
                        if (paddleTexturePosition.X < 840) paddleTexturePosition.X += 5;
                    if (GamePad.GetState(PlayerIndex.One).DPad.Left == ButtonState.Pressed)
                        if (paddleTexturePosition.X > 340) paddleTexturePosition.X -= 5;
                    ballTexturePosition += speed;
                    if (WallCheck()) break;
                    if (BlockCheck()) break;
                    PaddleCheck();
                    break;
            } 

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            spriteBatch.Begin();
            spriteBatch.Draw(backTexture, new Vector2(340, 0), Color.White);
            foreach (Vector2 block in blocks) spriteBatch.Draw(blockTexture, block, Color.White);
            spriteBatch.Draw(paddleTexture, paddleTexturePosition, Color.White);
            spriteBatch.Draw(ballTexture, ballTexturePosition, Color.White);
            spriteBatch.DrawString(font, "SCORE : " + score.ToString(), new Vector2(120, 650), Color.Black);
            if (gameState == START) spriteBatch.DrawString(font, "A ボ タ ン で ス タ ー ト ! !", new Vector2(500, 300), Color.White);
            spriteBatch.End(); 

            base.Draw(gameTime);
        }
        private bool WallCheck()
        {
            if (ballTexturePosition.Y < 0)
            {
                ballTexturePosition.Y = 0;
                speed.Y = -speed.Y;
            }
            else if (ballTexturePosition.X < 340)
            {
                ballTexturePosition.X = 340;
                speed.X = -speed.X;
            }
            else if (ballTexturePosition.X + 30 > 940)
            {
                ballTexturePosition.X = 910;
                speed.X = -speed.X;
            }
            else if (ballTexturePosition.Y > 720)
            {
                gameState = START;
                soundBank.GetCue("end").Play();
                return true;
            }
            return false;
        }

        private void PaddleCheck()
        {
            if ((ballTexturePosition.X + 30) > paddleTexturePosition.X &&
                (paddleTexturePosition.X + 100) > ballTexturePosition.X &&
                (ballTexturePosition.Y + 30) > paddleTexturePosition.Y &&
                (paddleTexturePosition.Y + 15) > (ballTexturePosition.Y + 30))
            {
                soundBank.GetCue("pon").Play();
                speed.X = new Random().Next(4) + 2;
                if (new Random().Next(2) == 0) speed.X = -speed.X;
                speed.Y = -(new Random().Next(3) + 2);
            }
        }


        private bool BlockCheck()
        {
            for (int i = 0; i < blocks.Count; i++)
            {
                BoundingBox blockBox = new BoundingBox(new Vector3(blocks[i].X, blocks[i].Y, 0),
                                                       new Vector3(blocks[i].X + 60, blocks[i].Y + 30, 0));
                BoundingBox ballBox = new BoundingBox(
                                       new Vector3(ballTexturePosition.X, ballTexturePosition.Y, 0),
                                       new Vector3(ballTexturePosition.X + 30, ballTexturePosition.Y + 30, 0));
                if (ballBox.Intersects(blockBox) == true)
                {
                    if (Math.Abs(blockBox.Max.X - ballBox.Min.X) < Math.Abs(speed.X))
                        speed.X = -speed.X;
                    else if (Math.Abs(blockBox.Min.X - ballBox.Max.X) < Math.Abs(speed.X))
                        speed.X = -speed.X;
                    else if (Math.Abs(blockBox.Max.Y - ballBox.Min.Y) < Math.Abs(speed.Y))
                        speed.Y = -speed.Y;
                    else 
                        speed.Y = -speed.Y;
                    score++;
                    blocks.Remove(blocks[i--]);
                    if (blocks.Count <= 0)
                    {
                        soundBank.GetCue("clear").Play();
                        gameState = START;
                        return true;
                    }
                    else
                    {
                        soundBank.GetCue("disa").Play();
                        return true;
                    }
                }
            }
            return false;
        }
    }
}
