﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Driving
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        enum State
        {
            Opening,
            Start,
            Running,
            Collision,
            Success
        }

        Texture2D carTexture;
        Texture2D roadTexture;
        Texture2D rockTexture;

        Vector2 carTexturePosition = new Vector2(490, 400);
        List<Vector2> rocks = new List<Vector2>();
        int[] roadTextureY = { 0, -720 };
        double rockSetTime = 1;
        int rocksPassed = 0;
        SpriteFont font;
        bool flg = false;
        State gameState = State.Opening;

        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank;
        Cue cue; 

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            audioEngine = new AudioEngine("Content/effectsound.xgs");
            waveBank = new WaveBank(audioEngine, "Content/Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, "Content/Sound Bank.xsb");

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here

            carTexture = Content.Load<Texture2D>("Graphics/car");
            roadTexture = Content.Load<Texture2D>("Graphics/road");
            rockTexture = Content.Load<Texture2D>("Graphics/rock");
            font = Content.Load<SpriteFont>("GameFont");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            switch (gameState)
            {
                case State.Opening:
                    if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed) gameState = State.Start;
                    break;
                case State.Start:
                    roadTextureY[0] = 0;
                    roadTextureY[1] = -720;
                    rocksPassed = 0;
                    rockSetTime = 1;
                    flg = false;
                    carTexturePosition = new Vector2(490, 400);
                    rocks.Clear();
                    gameState = State.Running;
                    cue = soundBank.GetCue("engine");
                    cue.Play();
                    break;
                case State.Running:
                    if (GamePad.GetState(PlayerIndex.One).DPad.Right == ButtonState.Pressed && flg == false)
                    {
                        flg = true;
                        carTexturePosition.X += 200;
                    }
                    else if (GamePad.GetState(PlayerIndex.One).DPad.Left == ButtonState.Pressed && flg == true)
                    {
                        flg = false;
                        carTexturePosition.X -= 200;
                    }
                    if (roadTextureY[0] >= 720)
                    {
                        roadTextureY[0] = 0;
                        roadTextureY[1] = -720;
                    }
                    for (int i = 0; i < roadTextureY.Length; i++) roadTextureY[i] += 4;
                    for (int i = 0; i < rocks.Count && gameState == State.Running; i++)
                    {
                        BoundingBox rockBox = new BoundingBox(new Vector3(rocks[i].X,rocks[i].Y, 0), 
                                                                new Vector3(rocks[i].X + 100,rocks[i].Y + 100, 0));
                        BoundingBox carBox = new BoundingBox(
                                             new Vector3(carTexturePosition.X, carTexturePosition.Y, 0),
                                             new Vector3(carTexturePosition.X + 100, carTexturePosition.Y + 100, 0));
                        if (rockBox.Intersects(carBox) == true)
                        {
                            cue.Stop(AudioStopOptions.AsAuthored);
                            soundBank.GetCue("collision").Play();
                            gameState = State.Collision;
                        }
                        else
                        {
                            rocks[i] = new Vector2(rocks[i].X, rocks[i].Y + 4);
                            if (rocks[i].Y > 720)
                            {
                                rocks.Remove(rocks[i--]);
                                if (++rocksPassed >= 10)
                                {
                                    gameState = State.Success;
                                    cue.Stop(AudioStopOptions.AsAuthored);
                                }
                            }
                        }
                    }
                    if ((rockSetTime -= gameTime.ElapsedGameTime.TotalSeconds) < 0)
                    {
                        rockSetTime = (double)new Random().Next(10, 20) / 10;
                        int aPosition = 490;
                        if (new Random().Next(1, 3) == 1) aPosition = 690;
                        rocks.Add(new Vector2(aPosition, -rockTexture.Height));
                    }
                    break;
                case State.Collision:
                    if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed) gameState = State.Start;
                    break;
                case State.Success:
                    if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed) gameState = State.Start;
                    break;
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            spriteBatch.Begin();
            spriteBatch.Draw(roadTexture, new Vector2(0, roadTextureY[0]), Color.White);
            spriteBatch.Draw(roadTexture, new Vector2(0, roadTextureY[1]), Color.White);
            foreach (Vector2 aHazard in rocks) spriteBatch.Draw(rockTexture, aHazard, Color.White);
            spriteBatch.Draw(carTexture, carTexturePosition, Color.White);
            spriteBatch.DrawString(font, "障 害 物 ク リ ア : " + rocksPassed.ToString(),
                                    new Vector2(10, 680), Color.Black);
            if (gameState == State.Opening)
                spriteBatch.DrawString(font, "A ボ タ ン で ス タ ー ト", new Vector2(502, 200), Color.Black);
            else if (gameState == State.Collision)
            {
                spriteBatch.DrawString(font, "衝 突 ! !", new Vector2(590, 200), Color.Black);
                spriteBatch.DrawString(font, "A ボ タ ン で 再 ス タ ー ト", new Vector2(482, 300), Color.Black);
            }
            else if (gameState == State.Success)
            {
                spriteBatch.DrawString(font, "ク リ ア お め で と う ! !", new Vector2(500, 200), Color.Black);
                spriteBatch.DrawString(font, "A ボ タ ン で 再 ス タ ー ト", new Vector2(482, 300), Color.Black);
            }
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
