﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Roulette
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Texture2D boardTexture;
        Texture2D messageTexture;
        Texture2D redTexture;
        Texture2D yellowTexture;
        Texture2D[] rouletteTexture = new Texture2D[6];
        Texture2D[] numberTexture = new Texture2D[6];
        Vector2[] numberXY = {  new Vector2(420, 130),new Vector2(570, 220),new Vector2(570, 400),
               new Vector2(420, 480),new Vector2(260, 400),new Vector2(260, 220)};
        const int START = 0;
        const int ROTATION = 1;
        const int STOP = 2;
        int gameState = START;

        GamePadState preGpState;
        int rouletteNumber;
        int betNumber;

        double startTime;

        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank; 

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
            IsFixedTimeStep = true;
            TargetElapsedTime = new TimeSpan(1666667); 
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            audioEngine = new AudioEngine("Content/pi.xgs");
            waveBank = new WaveBank(audioEngine, "Content/Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, "Content/Sound Bank.xsb");

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here

            boardTexture = Content.Load<Texture2D>("Graphics/board");
            messageTexture = Content.Load<Texture2D>("Graphics/message_1");
            redTexture = Content.Load<Texture2D>("Graphics/red");
            yellowTexture = Content.Load<Texture2D>("Graphics/yellow");
            for (int i = 0; i < numberTexture.Length; i++)
            {
                numberTexture[i] = Content.Load<Texture2D>("Graphics/number_" + (i + 1));
                if (i == 0) rouletteTexture[i] = Content.Load<Texture2D>("Graphics/red");
                else rouletteTexture[i] = Content.Load<Texture2D>("Graphics/green");
            }
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            switch (gameState)
            {
                case START:
                    GameStart(gameTime);
                    break;
                case ROTATION:
                    GameRotation(gameTime);
                    break;
                case STOP:
                    GameStop();
                    break;
            }
            preGpState = GamePad.GetState(PlayerIndex.One);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            spriteBatch.Begin();
            spriteBatch.Draw(boardTexture, new Vector2(760, 260), Color.White);
            spriteBatch.Draw(messageTexture, new Vector2(770, 310), Color.White);
            for (int i = 0; i < numberTexture.Length; i++)
            {
                spriteBatch.Draw(rouletteTexture[i], numberXY[i], Color.White);
                spriteBatch.Draw(numberTexture[i], numberXY[i], Color.White);
            }
            spriteBatch.End();

            base.Draw(gameTime);
        }
        private void GameStart(GameTime gameTime)
        {
            GamePadState GpState = GamePad.GetState(PlayerIndex.One);
            if (GpState.DPad.Right == ButtonState.Pressed && preGpState.DPad.Right == ButtonState.Released)
            {
                betNumber = (++betNumber) % 6;
                if (betNumber == 0) rouletteTexture[5] = Content.Load<Texture2D>("Graphics/green");
                else rouletteTexture[betNumber - 1] = Content.Load<Texture2D>("Graphics/green");
                rouletteTexture[betNumber] = Content.Load<Texture2D>("Graphics/red");
            }
            else if (GpState.DPad.Left == ButtonState.Pressed && preGpState.DPad.Left == ButtonState.Released)
            {
                betNumber--;
                if (betNumber < 0) betNumber = 5;
                if (betNumber == 5) rouletteTexture[0] = Content.Load<Texture2D>("Graphics/green");
                else rouletteTexture[betNumber + 1] = Content.Load<Texture2D>("Graphics/green");
                rouletteTexture[betNumber] = Content.Load<Texture2D>("Graphics/red");
            }
            else if (GpState.Buttons.A == ButtonState.Pressed)
            {
                startTime = gameTime.TotalGameTime.TotalMilliseconds + 5000 + (new Random().Next(6) * 100);
                gameState = ROTATION;
            }
        }

        private void GameRotation(GameTime gameTime)
        {
            rouletteNumber = (++rouletteNumber) % 6;
            if (rouletteNumber == 0)
            {
                if (betNumber == 5) rouletteTexture[5] = Content.Load<Texture2D>("Graphics/red");
                else rouletteTexture[5] = Content.Load<Texture2D>("Graphics/green");
            }
            else
            {
                if (betNumber == (rouletteNumber - 1))
                    rouletteTexture[rouletteNumber - 1] = Content.Load<Texture2D>("Graphics/red");
                else rouletteTexture[rouletteNumber - 1] = Content.Load<Texture2D>("Graphics/green");
            }
            rouletteTexture[rouletteNumber] = Content.Load<Texture2D>("Graphics/yellow");
            soundBank.GetCue("pi").Play();
            if (gameTime.TotalGameTime.TotalMilliseconds > startTime) gameState = STOP;
        }

        private void GameStop()
        {
            if (betNumber == rouletteNumber) messageTexture = Content.Load<Texture2D>("Graphics/message_2");
            else messageTexture = Content.Load<Texture2D>("Graphics/message_3");
            gameState = START;
        }
    }
}
